@@ -1,6 +1,6 @@
 /* Aravis - Digital camera library
  *
- * Copyright © 2009-2019 Emmanuel Pacaud
+ * Copyright © 2009-2022 Emmanuel Pacaud
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,7 +17,7 @@
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
- * Author: Emmanuel Pacaud <emmanuel@gnome.org>
+ * Author: Emmanuel Pacaud <emmanuel.pacaud@free.fr>
  */
 
 /**
@@ -66,7 +66,7 @@ G_DEFINE_ABSTRACT_TYPE_WITH_CODE (ArvDevice, arv_device, G_TYPE_OBJECT,
 				  G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE, arv_device_initable_iface_init))
 
 /**
- * arv_device_create_stream:
+ * arv_device_create_stream: (skip)
  * @device: a #ArvDevice
  * @callback: (scope call): a frame processing callback
  * @user_data: (allow-none) (closure): user data for @callback
@@ -82,10 +82,32 @@ G_DEFINE_ABSTRACT_TYPE_WITH_CODE (ArvDevice, arv_device, G_TYPE_OBJECT,
 
 ArvStream *
 arv_device_create_stream (ArvDevice *device, ArvStreamCallback callback, void *user_data, GError **error)
+{
+	return arv_device_create_stream_full(device, callback, user_data, NULL, error);
+}
+
+/**
+ * arv_device_create_stream_full: (rename-to arv_device_create_stream)
+ * @device: a #ArvDevice
+ * @callback: (scope notified): a frame processing callback
+ * @user_data: (allow-none) (closure): user data for @callback
+ * @destroy: a #GDestroyNotify placeholder, %NULL to ignore
+ * @error: a #GError placeholder, %NULL to ignore
+ *
+ * Creates a new #ArvStream for video stream handling. See
+ * @ArvStreamCallback for details regarding the callback function.
+ *
+ * Return value: (transfer full): a new #ArvStream.
+ *
+ * Since: 0.8.23
+ */
+
+ArvStream *
+arv_device_create_stream_full (ArvDevice *device, ArvStreamCallback callback, void *user_data, GDestroyNotify destroy, GError **error)
 {
 	g_return_val_if_fail (ARV_IS_DEVICE (device), NULL);
 
-	return ARV_DEVICE_GET_CLASS (device)->create_stream (device, callback, user_data, error);
+	return ARV_DEVICE_GET_CLASS (device)->create_stream (device, callback, user_data, destroy, error);
 }
 
 /**
@@ -323,6 +345,29 @@ arv_device_is_feature_available (ArvDevice *device, const char *feature, GError
 	return ARV_IS_GC_FEATURE_NODE (node) && arv_gc_feature_node_is_available (ARV_GC_FEATURE_NODE (node), error);
 }
 
+/**
+ * arv_device_is_feature_implemented:
+ * @device: a #ArvDevice
+ * @feature: feature name
+ * @error: a #GError placeholder, %NULL to ignore
+ *
+ * Return: %TRUE if feature is implemented, %FALSE if not or on error.
+ *
+ * Since: 0.8.23
+ */
+
+gboolean
+arv_device_is_feature_implemented (ArvDevice *device, const char *feature, GError **error)
+{
+	ArvGcNode* node;
+
+	g_return_val_if_fail (ARV_IS_DEVICE (device), FALSE);
+	g_return_val_if_fail (feature != NULL, FALSE);
+
+	node = arv_device_get_feature (device, feature);
+	return ARV_IS_GC_FEATURE_NODE (node) && arv_gc_feature_node_is_implemented (ARV_GC_FEATURE_NODE (node), error);
+}
+
 static void *
 _get_feature (ArvDevice *device, GType node_type, const char *feature, GError **error)
 {
@@ -923,20 +968,48 @@ arv_device_set_features_from_string (ArvDevice *device, const char *string, GErr
 			ArvGcNode *feature;
 			char *key = g_match_info_fetch_named (match_info, "Key");
 			char *value = g_match_info_fetch_named (match_info, "Value");
+                        size_t key_length = key != NULL ? strlen (key) : 0;
 
-			feature = arv_device_get_feature (device, key);
-			if (ARV_IS_GC_FEATURE_NODE (feature)) {
-				if (ARV_IS_GC_COMMAND (feature)) {
-					arv_device_execute_command (device, key, &local_error);
-				} else if (value != NULL) {
-					arv_gc_feature_node_set_value_from_string (ARV_GC_FEATURE_NODE (feature), value, &local_error);
-				} else {
-					g_set_error (&local_error, ARV_DEVICE_ERROR, ARV_DEVICE_ERROR_INVALID_PARAMETER,
-						     "[%s] Require a parameter value to set", key);
-				}
-			} else
-				g_set_error (&local_error, ARV_DEVICE_ERROR, ARV_DEVICE_ERROR_FEATURE_NOT_FOUND,
-					     "[%s] Not found", key);
+                        if (key_length > 4 && key[0] == 'R' && key[1] == '[' && key[key_length - 1] == ']') {
+                                char *end;
+                                gint64 address;
+                                gint64 int_value;
+
+                                address = g_ascii_strtoll (&key[2], &end, 0);
+                                if (end == NULL || end != key + key_length -1) {
+                                        g_set_error (&local_error,
+                                                     ARV_DEVICE_ERROR,
+                                                     ARV_DEVICE_ERROR_INVALID_PARAMETER,
+                                                     "Invalid address in %s", key);
+                                } else {
+                                        int_value = g_ascii_strtoll (value, &end, 0);
+                                        if (end == NULL || end[0] != '\0') {
+                                                g_set_error (&local_error,
+                                                             ARV_DEVICE_ERROR,
+                                                             ARV_DEVICE_ERROR_INVALID_PARAMETER,
+                                                             "Invalid %s value for %s", value, key);
+                                        } else {
+                                                arv_device_write_register (device, address, int_value, &local_error);
+                                        }
+                                }
+                        } else {
+                                feature = arv_device_get_feature (device, key);
+                                if (ARV_IS_GC_FEATURE_NODE (feature)) {
+                                        if (ARV_IS_GC_COMMAND (feature)) {
+                                                arv_device_execute_command (device, key, &local_error);
+                                        } else if (value != NULL) {
+                                                arv_gc_feature_node_set_value_from_string (ARV_GC_FEATURE_NODE (feature),
+                                                                                           value, &local_error);
+                                        } else {
+                                                g_set_error (&local_error,
+                                                             ARV_DEVICE_ERROR,
+                                                             ARV_DEVICE_ERROR_INVALID_PARAMETER,
+                                                             "[%s] Require a parameter value to set", key);
+                                        }
+                                } else
+                                        g_set_error (&local_error, ARV_DEVICE_ERROR, ARV_DEVICE_ERROR_FEATURE_NOT_FOUND,
+                                                     "[%s] Not found", key);
+                        }
 
 			g_free (key);
 			g_free (value);
