@@ -1,6 +1,6 @@
 /* Aravis - Digital camera library
  *
- * Copyright © 2009-2019 Emmanuel Pacaud
+ * Copyright © 2009-2022 Emmanuel Pacaud
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,7 +17,7 @@
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
- * Author: Emmanuel Pacaud <emmanuel@gnome.org>
+ * Author: Emmanuel Pacaud <emmanuel.pacaud@free.fr>
  */
 
 /**
@@ -55,7 +55,8 @@ enum {
 	ARV_STREAM_PROPERTY_EMIT_SIGNALS,
 	ARV_STREAM_PROPERTY_DEVICE,
 	ARV_STREAM_PROPERTY_CALLBACK,
-	ARV_STREAM_PROPERTY_CALLBACK_DATA
+	ARV_STREAM_PROPERTY_CALLBACK_DATA,
+	ARV_STREAM_PROPERTY_DESTROY_NOTIFY
 } ArvStreamProperties;
 
 typedef struct {
@@ -67,6 +68,7 @@ typedef struct {
 	ArvDevice *device;
 	ArvStreamCallback callback;
 	void *callback_data;
+	GDestroyNotify destroy_notify;
 
 	GError *init_error;
 
@@ -660,6 +662,9 @@ arv_stream_set_property (GObject * object, guint prop_id,
 		case ARV_STREAM_PROPERTY_CALLBACK_DATA:
 			priv->callback_data = g_value_get_pointer (value);
 			break;
+		case ARV_STREAM_PROPERTY_DESTROY_NOTIFY:
+			priv->destroy_notify = g_value_get_pointer (value);
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -759,6 +764,10 @@ arv_stream_finalize (GObject *object)
         g_ptr_array_foreach (priv->infos, (GFunc) arv_stream_info_free, NULL);
         g_clear_pointer (&priv->infos, g_ptr_array_unref);
 
+	if (priv->destroy_notify != NULL) {
+		priv->destroy_notify(priv->callback_data);
+	}
+
 	G_OBJECT_CLASS (arv_stream_parent_class)->finalize (object);
 }
 
@@ -824,6 +833,13 @@ arv_stream_class_init (ArvStreamClass *node_class)
 				       "Stream callback data",
 				       "Optional user callback data",
 				       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property
+		(object_class,
+		 ARV_STREAM_PROPERTY_DESTROY_NOTIFY,
+		 g_param_spec_pointer ("destroy-notify",
+				       "Destroy notify",
+				       "Optional destroy notify",
+				       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
 
 static gboolean
