@@ -1,6 +1,6 @@
 /* Aravis - Digital camera library
  *
- * Copyright © 2009-2021 Emmanuel Pacaud
+ * Copyright © 2009-2022 Emmanuel Pacaud
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,7 +17,7 @@
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
- * Author: Emmanuel Pacaud <emmanuel@gnome.org>
+ * Author: Emmanuel Pacaud <emmanuel.pacaud@free.fr>
  */
 
 #include <glib/gprintf.h>
@@ -419,6 +419,7 @@ arv_test_device_properties (ArvTest *test, const char *test_name, ArvTestCamera
         ArvTestStatus status;
         gint sensor_width, sensor_height;
         gsize size = 0;
+        gboolean available;
 
         g_return_if_fail (ARV_IS_TEST (test));
 
@@ -447,36 +448,108 @@ arv_test_device_properties (ArvTest *test, const char *test_name, ArvTestCamera
         arv_test_camera_add_result (test_camera, test_name, "SensorSizeCheck", status, comment);
 
         g_clear_error (&error);
-        arv_camera_get_gain (test_camera->camera, &error);
-        arv_test_camera_add_result (test_camera, test_name, "GainReadout",
-                                    error == NULL ? ARV_TEST_STATUS_SUCCESS : ARV_TEST_STATUS_FAILURE,
+        available = arv_camera_is_gain_available(test_camera->camera, &error);
+        arv_test_camera_add_result (test_camera, test_name, "GainAvailable",
+                                    error == NULL &&
+                                    available == arv_test_camera_get_key_file_boolean(test_camera, test,
+                                                                                      "GainAvailable",
+                                                                                      TRUE) ?
+                                    ARV_TEST_STATUS_SUCCESS : ARV_TEST_STATUS_FAILURE,
                                     error != NULL ? error->message : NULL);
 
+        if (available) {
+                g_clear_error (&error);
+                arv_camera_get_gain (test_camera->camera, &error);
+                arv_test_camera_add_result (test_camera, test_name, "GainReadout",
+                                            error == NULL ? ARV_TEST_STATUS_SUCCESS : ARV_TEST_STATUS_FAILURE,
+                                            error != NULL ? error->message : NULL);
+        }
+
         g_clear_error (&error);
-        arv_camera_get_exposure_time (test_camera->camera, &error);
-        arv_test_camera_add_result (test_camera, test_name, "ExposureTimeReadout",
-                                    error == NULL ? ARV_TEST_STATUS_SUCCESS : ARV_TEST_STATUS_FAILURE,
+        available = arv_camera_is_exposure_time_available (test_camera->camera, &error);
+        arv_test_camera_add_result (test_camera, test_name, "ExposureTimeAvailable",
+                                    error == NULL &&
+                                    available == arv_test_camera_get_key_file_boolean(test_camera, test,
+                                                                                      "ExposureTimeAvailable",
+                                                                                      TRUE) ?
+                                    ARV_TEST_STATUS_SUCCESS : ARV_TEST_STATUS_FAILURE,
                                     error != NULL ? error->message : NULL);
 
+        if (available) {
+                g_clear_error (&error);
+                arv_camera_get_exposure_time (test_camera->camera, &error);
+                arv_test_camera_add_result (test_camera, test_name, "ExposureTimeReadout",
+                                            error == NULL ? ARV_TEST_STATUS_SUCCESS : ARV_TEST_STATUS_FAILURE,
+                                            error != NULL ? error->message : NULL);
+        }
+
         g_clear_error (&error);
         g_free (sensor_size);
         g_free (comment);
 }
 
 static void
-_single_acquisition (ArvTest *test, const char *test_name, ArvTestCamera *test_camera, gboolean chunk_test)
+_single_acquisition (ArvTest *test, const char *test_name, ArvTestCamera *test_camera,
+                     gboolean chunk_test, gboolean multipart_test)
 {
         GError *error = NULL;
         ArvBuffer *buffer = NULL;
         char **chunk_list = NULL;
         ArvChunkParser *parser = NULL;
+        char *component = NULL;
+        guint n_parts = 1;
 
         g_return_if_fail (ARV_IS_TEST (test));
 
+        g_assert (!(multipart_test & chunk_test));
+
+        if (multipart_test) {
+                char  *multiparts = arv_test_camera_get_key_file_string (test_camera, test, "Multipart", NULL);
+                char **parts;
+                gboolean multipart_supported;
+                gboolean multipart_expected;
+                guint i;
+
+                parts = multiparts != NULL ? g_strsplit(multiparts, " ", -1) : NULL;
+                n_parts = parts != NULL ? g_strv_length(parts) : 0;
+
+                multipart_supported = arv_camera_gv_is_multipart_supported (test_camera->camera, &error);
+                multipart_expected = n_parts > 1;
+
+                if (!multipart_expected || (multipart_supported != multipart_expected)) {
+                        arv_test_camera_add_result (test_camera, test_name, "NoSupport",
+                                                    error == NULL && (multipart_expected == multipart_supported) ?
+                                                    ARV_TEST_STATUS_SUCCESS : ARV_TEST_STATUS_FAILURE,
+                                                    error != NULL ? error->message : NULL);
+                        g_clear_error (&error);
+                        return;
+                }
+
+                arv_camera_gv_set_multipart(test_camera->camera, TRUE, &error);
+                arv_test_camera_add_result (test_camera, test_name, "Enable",
+                                            error == NULL ? ARV_TEST_STATUS_SUCCESS : ARV_TEST_STATUS_FAILURE,
+                                            error != NULL ? error->message : NULL);
+                if (error != NULL) {
+                        g_clear_error (&error);
+                        return;
+                }
+
+                for (i = 0; i < n_parts && error == NULL; i++) {
+                        arv_camera_select_and_enable_component(test_camera->camera, parts[i], i == 0, &error);
+                }
+
+                component = g_strdup (parts[0]);
+
+                g_clear_pointer (&parts, g_strfreev);
+                g_free (multiparts);
+        }
+
         if (chunk_test) {
                 char *chunks;
                 gboolean chunks_support = arv_test_camera_get_key_file_boolean (test_camera, test,
                                                                                 "ChunksSupport", TRUE);
+                n_parts = arv_test_camera_get_key_file_int64 (test_camera, test,
+                                                              "ChunksNParts", 1);
 
                 if (!arv_camera_are_chunks_available (test_camera->camera, &error)) {
                         arv_test_camera_add_result (test_camera, test_name, "NoSupport",
@@ -500,23 +573,47 @@ _single_acquisition (ArvTest *test, const char *test_name, ArvTestCamera *test_c
         if (error == NULL)
                 buffer = arv_camera_acquisition (test_camera->camera, 1000000, &error);
 
-        if (chunk_test) {
+        if (error == NULL && (!ARV_IS_BUFFER (buffer) || arv_buffer_get_status(buffer) != ARV_BUFFER_STATUS_SUCCESS)) {
+                g_set_error (&error, ARV_DEVICE_ERROR, ARV_DEVICE_ERROR_TRANSFER_ERROR,
+                             "Buffer transfer failure");
+        }
+
+        if (error == NULL && ARV_IS_BUFFER (buffer)) {
+                if (arv_buffer_get_n_parts(buffer) != n_parts)
+                        g_set_error (&error, ARV_DEVICE_ERROR, ARV_DEVICE_ERROR_TRANSFER_ERROR,
+                                     "Invalid number of parts (found: %u - expected: %u)",
+                                     arv_buffer_get_n_parts(buffer), n_parts);
+        }
+
+        if (error == NULL && chunk_test) {
                 int n_chunks = g_strv_length (chunk_list);
+                int i;
 
-                if (ARV_IS_BUFFER (buffer)) {
-                        int i;
-
+                if (arv_buffer_has_chunks(buffer)) {
                         for (i = 0; i < n_chunks && error == NULL; i++) {
                                 char *chunk_name = g_strdup_printf ("Chunk%s", chunk_list[i]);
                                 arv_chunk_parser_get_integer_value (parser, buffer, chunk_name, &error);
                                 g_clear_pointer (&chunk_name, g_free);
                         }
+                } else {
+                        g_set_error (&error, ARV_DEVICE_ERROR, ARV_DEVICE_ERROR_TRANSFER_ERROR,
+                                     "No chunk found in buffer");
                 }
 
                 g_clear_object (&parser);
                 g_strfreev (chunk_list);
         }
 
+        if (chunk_test)
+                arv_camera_set_chunks (test_camera->camera, NULL, NULL);
+
+        if (error == NULL && multipart_test) {
+                arv_camera_gv_set_multipart (test_camera->camera, FALSE, &error);
+                if (error == NULL) arv_camera_select_and_enable_component(test_camera->camera, component, TRUE, &error);
+
+                g_free (component);
+        }
+
         arv_test_camera_add_result (test_camera, test_name, "BufferCheck",
                                     ARV_IS_BUFFER (buffer) &&
                                     arv_buffer_get_status (buffer) == ARV_BUFFER_STATUS_SUCCESS &&
@@ -531,13 +628,22 @@ _single_acquisition (ArvTest *test, const char *test_name, ArvTestCamera *test_c
 static void
 arv_test_single_acquisition (ArvTest *test, const char *test_name, ArvTestCamera *test_camera)
 {
-        _single_acquisition (test, test_name, test_camera, FALSE);
+        _single_acquisition (test, test_name, test_camera, FALSE, FALSE);
+}
+
+static void
+arv_test_multipart (ArvTest *test, const char  *test_name, ArvTestCamera *test_camera)
+{
+        if (!arv_camera_is_gv_device (test_camera->camera))
+                return;
+
+        _single_acquisition (test, test_name, test_camera, FALSE, TRUE);
 }
 
 static void
 arv_test_chunks (ArvTest *test, const char *test_name, ArvTestCamera *test_camera)
 {
-        _single_acquisition (test, test_name, test_camera, TRUE);
+        _single_acquisition (test, test_name, test_camera, TRUE, FALSE);
 }
 
 static void
@@ -580,7 +686,7 @@ _multiple_acquisition (ArvTest *test, const char *test_name, ArvTestCamera *test
                 for (i = 0 ; i < n_expected_buffers; i++) {
                         ArvBuffer *buffer;
 
-                        buffer = arv_stream_timeout_pop_buffer (stream, 500000);
+                        buffer = arv_stream_timeout_pop_buffer (stream, i == 0 ? 5000000 : 500000);
                         if (buffer == NULL)
                                 success = FALSE;
                         else {
@@ -606,13 +712,9 @@ _multiple_acquisition (ArvTest *test, const char *test_name, ArvTestCamera *test
 
         g_clear_object (&stream);
 
-        if (success) {
-                message = g_strdup_printf ("%u/%u", n_completed_buffers, n_expected_buffers);
-        } else {
-                message = g_strdup_printf ("%u/%u%s%s", n_completed_buffers, n_expected_buffers,
-                                           error != NULL ? " " : "",
-                                           error != NULL ? error->message : "");
-        }
+        message = g_strdup_printf ("%u/%u%s%s", n_completed_buffers, n_expected_buffers,
+                                   error != NULL ? " " : "",
+                                   error != NULL ? error->message : "");
 
         arv_test_camera_add_result (test_camera, test_name, "BufferCheck",
                                     success && error == NULL ? ARV_TEST_STATUS_SUCCESS : ARV_TEST_STATUS_FAILURE,
@@ -681,6 +783,7 @@ arv_test_software_trigger (ArvTest *test, const char *test_name, ArvTestCamera *
         gboolean software_trigger_support;
         guint n_completed_buffers = 0;
         guint n_expected_buffers = 5;
+        double delay_s;
 
         g_return_if_fail (ARV_IS_TEST (test));
 
@@ -696,6 +799,8 @@ arv_test_software_trigger (ArvTest *test, const char *test_name, ArvTestCamera *
                         return;
         }
 
+        delay_s = arv_test_camera_get_key_file_double (test_camera, test, "SoftwareTriggerWait", 0);
+
         arv_camera_set_acquisition_mode (test_camera->camera, ARV_ACQUISITION_MODE_CONTINUOUS, &error);
         if (error == NULL)
                 arv_camera_set_trigger (test_camera->camera, "Software", &error);
@@ -721,6 +826,8 @@ arv_test_software_trigger (ArvTest *test, const char *test_name, ArvTestCamera *
                 ArvBuffer *buffer;
 
                 if (error == NULL) {
+                        if (i > 0)
+                                g_usleep (1000000.0 * delay_s);
                         arv_camera_software_trigger (test_camera->camera, &error);
                 }
 
@@ -760,21 +867,42 @@ static void
 arv_test_gige_vision (ArvTest *test, const char *test_name, ArvTestCamera *test_camera)
 {
         GError *error = NULL;
-        gint64 expected_n_stream_channels;
-        gint64 n_stream_channels;
+        gint expected_n_stream_channels;
+        gint n_stream_channels;
+        gint expected_n_network_interfaces;
+        gint n_network_interfaces;
+        char *message;
 
         g_return_if_fail (ARV_IS_TEST (test));
 
         if (!arv_camera_is_gv_device (test_camera->camera))
                 return;
 
+        expected_n_network_interfaces = arv_test_camera_get_key_file_int64 (test_camera, test, "NNetworkInterfaces", 1);
+
+        n_network_interfaces = arv_camera_gv_get_n_network_interfaces (test_camera->camera, &error);
+
+        if (error != NULL)
+                message = g_strdup_printf("%s", error->message);
+        else
+                message = g_strdup_printf("%d", n_network_interfaces);
+        arv_test_camera_add_result (test_camera, test_name, "NNetworkInterfaces",
+                                    n_network_interfaces == expected_n_network_interfaces && error == NULL ?
+                                    ARV_TEST_STATUS_SUCCESS : ARV_TEST_STATUS_FAILURE, message);
+        g_clear_pointer(&message, g_free);
+
         expected_n_stream_channels = arv_test_camera_get_key_file_int64 (test_camera, test, "NStreamChannels", 1);
 
         n_stream_channels = arv_camera_gv_get_n_stream_channels (test_camera->camera, &error);
+
+        if (error != NULL)
+                message = g_strdup_printf("%s", error->message);
+        else
+                message = g_strdup_printf("%d", n_stream_channels);
         arv_test_camera_add_result (test_camera, test_name, "NStreamChannels",
                                     n_stream_channels == expected_n_stream_channels && error == NULL ?
-                                    ARV_TEST_STATUS_SUCCESS : ARV_TEST_STATUS_FAILURE,
-                                    error != NULL ? error->message : NULL);
+                                    ARV_TEST_STATUS_SUCCESS : ARV_TEST_STATUS_FAILURE, message);
+        g_clear_pointer(&message, g_free);
 
         g_clear_error (&error);
         if (expected_n_stream_channels > 0) {
@@ -806,9 +934,10 @@ const struct {
         {"Genicam",                     arv_test_genicam,               FALSE},
         {"Properties",                  arv_test_device_properties,     FALSE},
         {"MultipleAcquisitionA",        arv_test_multiple_acquisition_a,FALSE},
+        {"SingleAcquisition",           arv_test_single_acquisition,    FALSE},
         {"SoftwareTrigger",             arv_test_software_trigger,      FALSE},
         {"MultipleAcquisitionB",        arv_test_multiple_acquisition_b,FALSE},
-        {"SingleAcquisition",           arv_test_single_acquisition,    FALSE},
+        {"Multipart",                   arv_test_multipart,             FALSE},
         {"Chunks",                      arv_test_chunks,                FALSE},
         {"GigEVision",                  arv_test_gige_vision,           FALSE},
         {"USB3Vision",                  arv_test_usb3_vision,           FALSE}
@@ -819,7 +948,8 @@ arv_test_run (ArvTest *test, unsigned int n_iterations,
               const char *camera_selection,
               const char *test_selection,
 	      ArvUvUsbMode usb_mode,
-              gboolean cache_check)
+              gboolean cache_check,
+              gboolean packet_socket)
 {
         GRegex *camera_regex;
         GRegex *test_regex;
@@ -856,6 +986,13 @@ arv_test_run (ArvTest *test, unsigned int n_iterations,
 					if (arv_camera_is_uv_device (test_camera->camera))
 						arv_camera_uv_set_usb_mode (test_camera->camera, usb_mode);
 
+                                        if (arv_camera_is_gv_device(test_camera->camera))
+                                                arv_camera_gv_set_stream_options
+                                                        (test_camera->camera,
+                                                         packet_socket ?
+                                                         ARV_GV_STREAM_OPTION_NONE :
+                                                         ARV_GV_STREAM_OPTION_PACKET_SOCKET_DISABLED);
+
                                         for (j = 0; j < G_N_ELEMENTS (tests); j++) {
                                                 if (g_regex_match (test_regex, tests[j].name, 0, NULL)) {
 
@@ -865,10 +1002,8 @@ arv_test_run (ArvTest *test, unsigned int n_iterations,
                                                                 double delay;
 
                                                                 delay_name = g_strdup_printf ("%sDelay", tests[j].name);
-                                                                delay = arv_test_camera_get_key_file_double (test_camera,
-                                                                                                             test,
-                                                                                                             delay_name,
-                                                                                                             0);
+                                                                delay = arv_test_camera_get_key_file_double
+                                                                        (test_camera, test, delay_name, 0);
                                                                 g_usleep (1000000.0 * delay);
                                                                 tests[j].run (test, tests[j].name, test_camera);
                                                                 g_free (delay_name);
@@ -882,6 +1017,7 @@ arv_test_run (ArvTest *test, unsigned int n_iterations,
                                                                 comment = arv_test_camera_get_key_file_comment
                                                                         (test_camera, test,
                                                                          tests[j].name);
+
                                                                 if (comment != NULL) {
                                                                         printf ("%s\n", comment);
                                                                         g_free (comment);
@@ -899,7 +1035,8 @@ arv_test_run (ArvTest *test, unsigned int n_iterations,
                                         n_cache_errors = arv_test_camera_get_n_register_cache_errors (test_camera);
 
                                         if (n_cache_errors > 0)
-                                                comment = g_strdup_printf ("%" G_GUINT64_FORMAT " error(s)", n_cache_errors);
+                                                comment = g_strdup_printf ("%" G_GUINT64_FORMAT " error(s)",
+                                                                           n_cache_errors);
 
                                         arv_test_camera_add_result (test_camera, "Genicam", "RegisterCache",
                                                                     n_cache_errors == 0 ?
@@ -939,6 +1076,8 @@ static char *arv_option_configuration = NULL;
 static char *arv_option_debug_domains = NULL;
 static char *arv_option_uv_usb_mode = NULL;
 static gboolean arv_option_cache_check = FALSE;
+static gboolean arv_option_packet_socket = FALSE;
+static gboolean arv_option_show_version = FALSE;
 
 static const GOptionEntry arv_option_entries[] =
 {
@@ -972,15 +1111,25 @@ static const GOptionEntry arv_option_entries[] =
 		&arv_option_cache_check,		"Register cache check",
 		NULL
         },
+	{
+		"packet-socket",			'\0', 0, G_OPTION_ARG_NONE,
+		&arv_option_packet_socket,		"Enable use of packet socket",
+		NULL
+	},
 	{
 		"debug", 				'd', 0, G_OPTION_ARG_STRING,
 		&arv_option_debug_domains, 		NULL,
 		"{<category>[:<level>][,...]|help}"
 	},
+	{
+		"version", 			        'v', 0, G_OPTION_ARG_NONE,
+		&arv_option_show_version,     	        "Show version",
+                NULL
+	},
 	{ NULL }
 };
 
-static const char *summary =
+static const char *description_content =
 "arv-test is an automated test utility that tries to exercise most of the\n"
 "Aravis functionalities. By default it runs all the tests on all the detected\n"
 "devices, but devices and tests can be selected using a glob pattern.\n\n"
@@ -997,7 +1146,8 @@ main (int argc, char **argv)
 	ArvUvUsbMode usb_mode;
 
 	context = g_option_context_new (NULL);
-        g_option_context_set_summary (context, summary);
+        g_option_context_set_summary (context, "Automated test utillity.");
+        g_option_context_set_description (context, description_content);
 	g_option_context_add_main_entries (context, arv_option_entries, NULL);
 
 	if (!g_option_context_parse (context, &argc, &argv, &error)) {
@@ -1008,6 +1158,15 @@ main (int argc, char **argv)
 	}
 
 	g_option_context_free (context);
+
+        if (arv_option_show_version) {
+                printf ("%u.%u.%u\n",
+                        arv_get_major_version (),
+                        arv_get_minor_version (),
+                        arv_get_micro_version ());
+                return EXIT_SUCCESS;
+        }
+
 	if (!arv_debug_enable (arv_option_debug_domains)) {
 		if (g_strcmp0 (arv_option_debug_domains, "help") != 0)
 			printf ("Invalid debug selection\n");
@@ -1043,7 +1202,8 @@ main (int argc, char **argv)
                            arv_option_camera_selection,
                            arv_option_test_selection,
                            usb_mode,
-                           arv_option_cache_check))
+                           arv_option_cache_check,
+                           arv_option_packet_socket))
                 success = FALSE;
 
         g_clear_object (&test);
